@@ -11,7 +11,7 @@ import warnings
 from pathlib import Path
 from typing import Dict, Iterable, List, Set, Tuple, Union
 
-import requests.exceptions
+from rnalysis.utils.param_typing import GO_ASPECTS, GO_EVIDENCE_TYPES, GO_QUALIFIERS, DEFAULT_ORGANISMS, GENE_ID_TYPES
 
 try:
     from typing import Literal
@@ -28,25 +28,6 @@ import upsetplot
 from rnalysis.filtering import Filter
 from rnalysis.utils import io, parsing, settings, validation, enrichment_runner, generic
 
-_ASPECTS = ('biological_process', 'molecular function', 'cellular component')
-_EVIDENCE_TYPES = ('experimental', 'phylogenetic', 'computational', 'author', 'curator', 'electronic')
-_QUALIFIERS = ('not', 'contributes_to', 'colocalizes_with')
-_DEFAULT_ORGANISMS = tuple(sorted(['Caenorhabditis elegans',
-                                   'Mus musculus',
-                                   'Drosophila melanogaster',
-                                   'Homo sapiens',
-                                   'Arabodopsis thaliana',
-                                   'Danio rerio',
-                                   'Escherichia coli',
-                                   'Saccharomyces cerevisiae',
-                                   'Schizosaccharomyces pombe']))
-try:
-    _GENE_ID_TYPES = parsing.data_to_tuple(io.get_legal_gene_id_types()[0].keys())
-
-
-except requests.exceptions.ConnectionError:
-    _GENE_ID_TYPES = tuple()
-
 
 class FeatureSet:
     """ Receives a filtered gene set and the set's name (optional) and preforms various enrichment analyses on them. """
@@ -268,21 +249,22 @@ class FeatureSet:
         """
         return FeatureSet(self._set_ops((other,), set.symmetric_difference))
 
-    def go_enrichment(self, organism: Union[str, int, Literal['auto'], Literal[_DEFAULT_ORGANISMS]] = 'auto',
-                      gene_id_type: Union[str, Literal['auto'], Literal[_GENE_ID_TYPES]] = 'auto', alpha: float = 0.05,
+    def go_enrichment(self, organism: Union[str, int, Literal['auto'], Literal[DEFAULT_ORGANISMS]] = 'auto',
+                      gene_id_type: Union[str, Literal['auto'], Literal[GENE_ID_TYPES]] = 'auto', alpha: float = 0.05,
                       statistical_test: Literal['fisher', 'hypergeometric', 'randomization'] = 'fisher',
                       biotype: Union[str, List[str], Literal['all']] = 'all',
                       background_genes: Union[Set[str], Filter, 'FeatureSet'] = None,
                       biotype_ref_path: Union[str, Path, Literal['predefined']] = 'predefined',
                       propagate_annotations: Literal['classic', 'elim', 'weight', 'all.m', 'no'] = 'elim',
-                      aspects: Union[Literal[('any',) + _ASPECTS], Iterable[Literal[_ASPECTS]]] = 'any',
+                      aspects: Union[Literal[('any',) + GO_ASPECTS], Iterable[Literal[GO_ASPECTS]]] = 'any',
                       evidence_types: Union[
-                          Literal[('any',) + _EVIDENCE_TYPES], Iterable[Literal[_EVIDENCE_TYPES]]] = 'any',
-                      excluded_evidence_types: Union[Literal[_EVIDENCE_TYPES], Iterable[Literal[_EVIDENCE_TYPES]]] = (),
+                          Literal[('any',) + GO_EVIDENCE_TYPES], Iterable[Literal[GO_EVIDENCE_TYPES]]] = 'any',
+                      excluded_evidence_types: Union[Literal[GO_EVIDENCE_TYPES], Iterable[Literal[
+                          GO_EVIDENCE_TYPES]]] = (),
                       databases: Union[str, Iterable[str]] = 'any',
                       excluded_databases: Union[str, Iterable[str]] = (),
-                      qualifiers: Union[Literal[('any',) + _QUALIFIERS], Iterable[Literal[_QUALIFIERS]]] = 'any',
-                      excluded_qualifiers: Union[Literal[_QUALIFIERS], Iterable[Literal[_QUALIFIERS]]] = 'not',
+                      qualifiers: Union[Literal[('any',) + GO_QUALIFIERS], Iterable[Literal[GO_QUALIFIERS]]] = 'any',
+                      excluded_qualifiers: Union[Literal[GO_QUALIFIERS], Iterable[Literal[GO_QUALIFIERS]]] = 'not',
                       return_nonsignificant: bool = False,
                       save_csv: bool = False, fname=None, return_fig: bool = False, plot_horizontal: bool = True,
                       plot_ontology_graph: bool = True, ontology_graph_format: Literal['pdf', 'svg', 'png'] = 'pdf',
@@ -395,21 +377,21 @@ class FeatureSet:
         :return: a pandas DataFrame with GO terms as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  ontology_graph.png
+        .. figure:: /figures/ontology_graph.png
            :align:   center
            :scale: 60 %
 
            Example plot of go_enrichment(plot_ontology_graph=True)
 
 
-        .. figure::  plot_enrichment_results_go.png
+        .. figure:: /figures/plot_enrichment_results_go.png
            :align:   center
            :scale: 60 %
 
            Example plot of go_enrichment()
 
 
-        .. figure::  plot_enrichment_results_go_vertical.png
+        .. figure:: /figures/plot_enrichment_results_go_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -435,8 +417,8 @@ class FeatureSet:
             return runner.run(plot=False), runner
         return runner.run()
 
-    def kegg_enrichment(self, organism: Union[str, int, Literal['auto'], Literal[_DEFAULT_ORGANISMS]] = 'auto',
-                        gene_id_type: Union[str, Literal['auto'], Literal[_GENE_ID_TYPES]] = 'auto',
+    def kegg_enrichment(self, organism: Union[str, int, Literal['auto'], Literal[DEFAULT_ORGANISMS]] = 'auto',
+                        gene_id_type: Union[str, Literal['auto'], Literal[GENE_ID_TYPES]] = 'auto',
                         alpha: float = 0.05,
                         statistical_test: Literal['fisher', 'hypergeometric', 'randomization'] = 'fisher',
                         biotype: Union[str, List[str], Literal['all']] = 'all',
@@ -512,14 +494,14 @@ class FeatureSet:
         and a matplotlib Figure, if 'return_figure' is set to True.
 
 
-        .. figure::  plot_enrichment_results_kegg.png
+        .. figure:: /figures/plot_enrichment_results_kegg.png
            :align:   center
            :scale: 60 %
 
            Example plot of kegg_enrichment()
 
 
-        .. figure::  plot_enrichment_results_kegg_vertical.png
+        .. figure:: /figures/plot_enrichment_results_kegg_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -615,14 +597,14 @@ class FeatureSet:
         :return: a pandas DataFrame with the indicated attribute names as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  plot_enrichment_results.png
+        .. figure:: /figures/plot_enrichment_results.png
            :align:   center
            :scale: 60 %
 
            Example plot of enrich_randomization()
 
 
-        .. figure::  plot_enrichment_results_vertical.png
+        .. figure:: /figures/plot_enrichment_results_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -714,14 +696,14 @@ class FeatureSet:
         :return: a pandas DataFrame with the indicated attribute names as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  plot_enrichment_results.png
+        .. figure:: /figures/plot_enrichment_results.png
            :align:   center
            :scale: 60 %
 
            Example plot of user_defined_enrichment()
 
 
-        .. figure::  plot_enrichment_results_vertical.png
+        .. figure:: /figures/plot_enrichment_results_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -813,14 +795,14 @@ class FeatureSet:
         :return: a pandas DataFrame with the indicated attribute names as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  plot_enrichment_results.png
+        .. figure:: /figures/plot_enrichment_results.png
            :align:   center
            :scale: 60 %
 
            Example plot of enrich_hypergeometric()
 
 
-        .. figure::  plot_enrichment_results_vertical.png
+        .. figure:: /figures/plot_enrichment_results_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -901,13 +883,13 @@ class FeatureSet:
         and a matplotlib Figure, if 'return_figure' is set to True.
 
 
-        .. figure::  hist_overlap.png
+        .. figure:: /figures/hist_overlap.png
            :align:   center
            :scale: 60 %
 
            Example plot of non_categorical_enrichment(plot_style`='overlap')
 
-        .. figure::  hist_interleaved.png
+        .. figure:: /figures/hist_interleaved.png
            :align:   center
            :scale: 60 %
 
@@ -1015,21 +997,22 @@ class RankedSet(FeatureSet):
                       "the return type will always be FeatureSet and not RankedSet.")
         return super()._set_ops(others, op)
 
-    def single_set_go_enrichment(self, organism: Union[str, int, Literal['auto'], Literal[_DEFAULT_ORGANISMS]] = 'auto',
-                                 gene_id_type: Union[str, Literal['auto'], Literal[_GENE_ID_TYPES]] = 'auto',
+    def single_set_go_enrichment(self, organism: Union[str, int, Literal['auto'], Literal[DEFAULT_ORGANISMS]] = 'auto',
+                                 gene_id_type: Union[str, Literal['auto'], Literal[GENE_ID_TYPES]] = 'auto',
                                  alpha: float = 0.05,
                                  propagate_annotations: Literal['classic', 'elim', 'weight', 'all.m', 'no'] = 'elim',
-                                 aspects: Union[Literal[('any',) + _ASPECTS], Iterable[Literal[_ASPECTS]]] = 'any',
+                                 aspects: Union[Literal[('any',) + GO_ASPECTS], Iterable[Literal[GO_ASPECTS]]] = 'any',
                                  evidence_types: Union[
-                                     Literal[('any',) + _EVIDENCE_TYPES], Iterable[Literal[_EVIDENCE_TYPES]]] = 'any',
+                                     Literal[('any',) + GO_EVIDENCE_TYPES], Iterable[Literal[
+                                         GO_EVIDENCE_TYPES]]] = 'any',
                                  excluded_evidence_types: Union[
-                                     Literal[_EVIDENCE_TYPES], Iterable[Literal[_EVIDENCE_TYPES]]] = (),
+                                     Literal[GO_EVIDENCE_TYPES], Iterable[Literal[GO_EVIDENCE_TYPES]]] = (),
                                  databases: Union[str, Iterable[str]] = 'any',
                                  excluded_databases: Union[str, Iterable[str]] = (),
                                  qualifiers: Union[
-                                     Literal[('any',) + _QUALIFIERS], Iterable[Literal[_QUALIFIERS]]] = 'any',
+                                     Literal[('any',) + GO_QUALIFIERS], Iterable[Literal[GO_QUALIFIERS]]] = 'any',
                                  excluded_qualifiers: Union[
-                                     Literal[_QUALIFIERS], Iterable[Literal[_QUALIFIERS]]] = 'not',
+                                     Literal[GO_QUALIFIERS], Iterable[Literal[GO_QUALIFIERS]]] = 'not',
                                  return_nonsignificant: bool = False,
                                  save_csv: bool = False, fname=None,
                                  return_fig: bool = False, plot_horizontal: bool = True,
@@ -1127,21 +1110,21 @@ class RankedSet(FeatureSet):
         :return: a pandas DataFrame with the indicated attribute names as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  ontology_graph_singlelist.png
+        .. figure:: /figures/ontology_graph_singlelist.png
            :align:   center
            :scale: 60 %
 
            Example plot of single_set_go_enrichment(plot_ontology_graph=True)
 
 
-        .. figure::  plot_enrichment_results_go_singlelist.png
+        .. figure:: /figures/plot_enrichment_results_go_singlelist.png
            :align:   center
            :scale: 60 %
 
            Example plot of single_set_go_enrichment()
 
 
-        .. figure::  plot_enrichment_results_go_singlelist_vertical.png
+        .. figure:: /figures/plot_enrichment_results_go_singlelist_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -1161,8 +1144,8 @@ class RankedSet(FeatureSet):
         return runner.run()
 
     def single_set_kegg_enrichment(self,
-                                   organism: Union[str, int, Literal['auto'], Literal[_DEFAULT_ORGANISMS]] = 'auto',
-                                   gene_id_type: Union[str, Literal['auto'], Literal[_GENE_ID_TYPES]] = 'auto',
+                                   organism: Union[str, int, Literal['auto'], Literal[DEFAULT_ORGANISMS]] = 'auto',
+                                   gene_id_type: Union[str, Literal['auto'], Literal[GENE_ID_TYPES]] = 'auto',
                                    alpha: float = 0.05, return_nonsignificant: bool = False, save_csv: bool = False,
                                    fname=None, return_fig: bool = False, plot_horizontal: bool = True,
                                    plot_pathway_graphs: bool = True, pathway_graphs_format: str = 'pdf',
@@ -1214,21 +1197,21 @@ class RankedSet(FeatureSet):
         :return: a pandas DataFrame with the indicated attribute names as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  pathway_graph_singlelist.png
+        .. figure:: /figures/pathway_graph_singlelist.png
            :align:   center
            :scale: 60 %
 
            Example plot of single_set_kegg_enrichment(plot_pathway_graphs=True)
 
 
-        .. figure::  plot_enrichment_results_kegg_single_set.png
+        .. figure:: /figures/plot_enrichment_results_kegg_single_set.png
            :align:   center
            :scale: 60 %
 
            Example plot of single_set_kegg_enrichment()
 
 
-        .. figure::  plot_enrichment_results_kegg_single_set_vertical.png
+        .. figure:: /figures/plot_enrichment_results_kegg_single_set_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -1292,14 +1275,14 @@ class RankedSet(FeatureSet):
         :return: a pandas DataFrame with the indicated attribute names as rows/index; \
         and a matplotlib Figure, if 'return_figure' is set to True.
 
-        .. figure::  plot_enrichment_results_single_set.png
+        .. figure:: /figures/plot_enrichment_results_single_set.png
            :align:   center
            :scale: 60 %
 
            Example plot of single_set_enrichment()
 
 
-        .. figure::  plot_enrichment_results_single_set_vertical.png
+        .. figure:: /figures/plot_enrichment_results_single_set_vertical.png
            :align:   center
            :scale: 60 %
 
@@ -1420,7 +1403,7 @@ def upset_plot(objs: Dict[str, Union[str, FeatureSet, Set[str]]], set_colors: It
     :returns: a dictionary of matplotlib axes, where the keys are 'matrix', 'intersections', 'totals', 'shading'.
 
 
-        .. figure::  upsetplot.png
+        .. figure:: /figures/upsetplot.png
            :align:   center
            :scale: 70 %
 
@@ -1480,7 +1463,7 @@ def _get_tuple_patch_ids(n_sets: int) -> List[Tuple[int, ...]]:
     return sorted_ids
 
 
-def venn_diagram(objs: Dict[str, Union[str, FeatureSet, Set[str]]], title: str = 'default',
+def venn_diagram(objs: Dict[str, Union[str, FeatureSet, Set[str]]], title: Union[str, Literal['default']] = 'default',
                  attr_ref_table_path: Union[str, Path, Literal['predefined']] = 'predefined',
                  set_colors: Iterable[str] = ('r', 'g', 'b'),
                  transparency: float = 0.4, weighted: bool = True, add_outline: bool = True, linecolor: str = 'black',
@@ -1494,7 +1477,7 @@ def venn_diagram(objs: Dict[str, Union[str, FeatureSet, Set[str]]], title: str =
     a FeatureSet, a python set of feature indices, or a name of a column in the Attribute Reference Table. \
     For example: \
     {'first set':{'gene1','gene2','gene3'}, 'second set':'name_of_attribute_from_reference_table'}
-    :type title: str
+    :type title: str or 'default' (default='default')
     :param set_colors: determines the colors of the circles in the diagram.
     :param attr_ref_table_path: the path of the Attribute Reference Table from which user-defined attributes will be drawn, \
     if such attributes are included in 'objs'.
@@ -1529,7 +1512,7 @@ def venn_diagram(objs: Dict[str, Union[str, FeatureSet, Set[str]]], title: str =
     :return: a tuple of a VennDiagram object; and a list of 2-3 Circle patches.
 
 
-        .. figure::  venn.png
+        .. figure:: /figures/venn.png
            :align:   center
            :scale: 70 %
 
