@@ -7,6 +7,7 @@ from pathlib import Path
 from queue import Queue
 
 import matplotlib
+import pandas as pd
 import typing_extensions
 from PyQt5 import QtCore, QtWidgets, QtGui
 from joblib import Parallel
@@ -183,8 +184,8 @@ class TableColumnGroupPicker(TableColumnPicker):
 
     def _get_groups_in_use(self):
         existing_groups = set()
-        for combo, col_name in zip(self.column_combos, self.columns):
-            if combo.isEnabled():
+        for check, combo, col_name in zip(self.column_checks,self.column_combos, self.columns):
+            if check.isChecked():
                 grp = combo.currentText()
                 existing_groups.add(grp)
         return sorted(existing_groups)
@@ -253,7 +254,7 @@ class PathInputDialog(QtWidgets.QDialog):
 
 
 class Worker(QtCore.QObject):
-    finished = QtCore.pyqtSignal(object, object, str)
+    finished = QtCore.pyqtSignal(tuple)
     startProgBar = QtCore.pyqtSignal(object)
 
     def __init__(self, partial, *args):
@@ -262,9 +263,16 @@ class Worker(QtCore.QObject):
         super().__init__()
 
     def run(self):
-        result = self.partial()
+        args = []
+        try:
+            result = self.partial()
+        except Exception as e:
+            self.finished.emit((e,))
+            return
+
         if result is not None:
-            self.finished.emit(*result, *self.args)
+            args = self.args
+        self.finished.emit((result, *args))
 
 
 class AltTQDM(QtCore.QObject):
@@ -331,6 +339,31 @@ class AltParallel(QtCore.QObject):
                 self.barFinished.emit()
 
 
+class TextWithCopyButton(QtWidgets.QWidget):
+    def __init__(self, text: str, parent=None):
+        super().__init__(parent)
+        self.text = text
+        self.text_edit = QtWidgets.QTextBrowser(self)
+        self.copy_button = QtWidgets.QPushButton('Copy to clipboard')
+        self.copied_label = QtWidgets.QLabel()
+        self.layout = QtWidgets.QVBoxLayout(self)
+        self.init_ui()
+
+    def init_ui(self):
+        self.text_edit.setHtml(self.text)
+        self.text_edit.setReadOnly(True)
+        self.layout.addWidget(self.text_edit)
+        self.copy_button.clicked.connect(self.copy_to_clipboard)
+        self.layout.addWidget(self.copy_button)
+        self.layout.addWidget(self.copied_label)
+
+    def copy_to_clipboard(self):
+        cb = QtWidgets.QApplication.clipboard()
+        cb.clear(mode=cb.Clipboard)
+        cb.setText(self.text_edit.toPlainText())
+        self.copied_label.setText('Copied to clipboard')
+
+
 class ToggleSwitchCore(QtWidgets.QPushButton):
     """
     Based upon a StackOverflow response by the user Heike:
@@ -539,14 +572,25 @@ class MultipleChoiceList(QtWidgets.QWidget):
         self.clear_all_button.clicked.connect(self.clear_all)
         self.layout.addWidget(self.clear_all_button, 6, 1)
 
+        self.current_layout_row = 6
+
         for row in range(2, 4):
             self.layout.setRowStretch(row, 2)
 
         self.itemSelectionChanged = self.list.itemSelectionChanged
-        self.selectedItems = self.list.selectedItems
 
         self.add_items(items, icons)
 
+    def get_sorted_selection(self):
+        items = self.list.selectedItems()
+        return sorted(items, key=self.list.row)
+
+    def get_sorted_selected_names(self):
+        return [name for name in self.items if name in {item.text() for item in self.list.selectedItems()}]
+
+    def get_sorted_names(self):
+        return self.items
+
     def add_item(self, item, icon=None):
         self.items.append(item)
         list_item = QtWidgets.QListWidgetItem(item)
@@ -580,8 +624,9 @@ class MultiChoiceListWithDelete(MultipleChoiceList):
         self.delete_all_button = QtWidgets.QPushButton('Delete all')
         self.delete_all_button.clicked.connect(self.delete_all)
 
-        self.layout.addWidget(self.delete_button, 7, 1)
-        self.layout.addWidget(self.delete_all_button, 8, 1)
+        self.layout.addWidget(self.delete_button, self.current_layout_row + 1, 1)
+        self.layout.addWidget(self.delete_all_button, self.current_layout_row + 2, 1)
+        self.current_layout_row += 2
 
     def delete_selected(self):
         for item in self.list.selectedItems():
@@ -603,6 +648,121 @@ class MultiChoiceListWithDelete(MultipleChoiceList):
             self.list.clear()
 
 
+class MultiChoiceListWithReorder(MultipleChoiceList):
+    itemOrderChanged = QtCore.pyqtSignal()
+
+    def __init__(self, items: typing.Sequence, icons: typing.Sequence = None, parent=None):
+        super().__init__(items, icons, parent)
+        self.top_button = QtWidgets.QPushButton('Top')
+        self.top_button.clicked.connect(self.top_selected)
+
+        self.up_button = QtWidgets.QPushButton('Up')
+        self.up_button.clicked.connect(self.up_selected)
+
+        self.down_button = QtWidgets.QPushButton('Down')
+        self.down_button.clicked.connect(self.down_selected)
+
+        self.bottom_button = QtWidgets.QPushButton('Bottom')
+        self.bottom_button.clicked.connect(self.bottom_selected)
+
+        self.reorder_label = QtWidgets.QLabel('<b>Reorder:</b>')
+        self.reorder_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignHCenter)
+        self.layout.addWidget(self.reorder_label, self.current_layout_row + 1, 1)
+        self.layout.addWidget(self.top_button, self.current_layout_row + 2, 1)
+        self.layout.addWidget(self.up_button, self.current_layout_row + 3, 1)
+        self.layout.addWidget(self.down_button, self.current_layout_row + 4, 1)
+        self.layout.addWidget(self.bottom_button, self.current_layout_row + 5, 1)
+        self.current_layout_row += 5
+
+    def up_selected(self):
+        for item in self.get_sorted_selection():
+            row = self.list.row(item)
+            if row > 0:
+                self.items[row - 1], self.items[row] = self.items[row], self.items[row - 1]
+                self.list_items[row - 1], self.list_items[row] = self.list_items[row], self.list_items[row - 1]
+                current_item = self.list.takeItem(row)
+                self.list.insertItem(row - 1, current_item)
+                current_item.setSelected(True)
+        self.itemOrderChanged.emit()
+
+    def down_selected(self):
+        for item in reversed(self.get_sorted_selection()):
+            row = self.list.row(item)
+            end = len(self.items) - 1
+            if row < end:
+                self.items[row + 1], self.items[row] = self.items[row], self.items[row + 1]
+                self.list_items[row + 1], self.list_items[row] = self.list_items[row], self.list_items[row + 1]
+                current_item = self.list.takeItem(row)
+                self.list.insertItem(row + 1, current_item)
+                current_item.setSelected(True)
+        self.itemOrderChanged.emit()
+
+    def top_selected(self):
+        for i, item in enumerate(self.get_sorted_selection()):
+            row = self.list.row(item)
+            if row > i:
+                self.items.insert(i, self.items.pop(row))
+                self.list_items.insert(i, self.list_items.pop(row))
+                current_item = self.list.takeItem(row)
+                self.list.insertItem(i, current_item)
+                current_item.setSelected(True)
+        self.itemOrderChanged.emit()
+
+    def bottom_selected(self):
+        for i, item in enumerate(reversed(self.get_sorted_selection())):
+            row = self.list.row(item)
+            this_end = len(self.items) - 1 - i
+            if row < this_end:
+                self.items.insert(this_end, self.items.pop(row))
+                self.list_items.insert(this_end, self.list_items.pop(row))
+
+                current_item = self.list.takeItem(row)
+                self.list.insertItem(this_end, current_item)
+                current_item.setSelected(True)
+        self.itemOrderChanged.emit()
+
+
+class MultiChoiceListWithDeleteReorder(MultiChoiceListWithReorder, MultiChoiceListWithDelete):
+    itemDeleted = QtCore.pyqtSignal(int)
+
+    def __init__(self, items: typing.Sequence, icons: typing.Sequence = None, parent=None):
+        super().__init__(items, icons, parent)
+
+
+class FileListWidgetItem(QtWidgets.QListWidgetItem):
+    def __init__(self, file_path, parent=None):
+        self.file_path = file_path
+        self.display_name = Path(file_path).stem
+        super().__init__(self.display_name, parent)
+
+    def filePath(self):
+        return self.file_path
+
+
+class OrderedFileList(MultiChoiceListWithDeleteReorder):
+    def __init__(self, parent=None):
+        super().__init__([], None, parent)
+        self.add_files_button = QtWidgets.QPushButton('Add files...', self)
+        self.add_files_button.clicked.connect(self.add_files)
+        self.layout.addWidget(self.add_files_button, 0, 1)
+
+    def add_files(self):
+        filenames, _ = QtWidgets.QFileDialog.getOpenFileNames(self, "Load fastq files")
+        if filenames:
+            self.add_items(filenames)
+
+    def add_item(self, item, icon=None):
+        self.items.append(item)
+        list_item = FileListWidgetItem(item)
+        if icon is not None:
+            list_item.setIcon(icon)
+        self.list_items.append(list_item)
+        self.list.addItem(list_item)
+
+    def get_sorted_names(self):
+        return [name for name in self.items]
+
+
 class MandatoryComboBox(QtWidgets.QComboBox):
     def __init__(self, default_choice: str, parent=None):
         super().__init__(parent)
@@ -694,10 +854,11 @@ class PathLineEdit(QtWidgets.QWidget):
     IS_LINE_EDIT_LIKE = True
     textChanged = QtCore.pyqtSignal(bool)
 
-    def __init__(self, contents: str = 'No file chosen', button_text: str = 'Load', parent=None):
+    def __init__(self, contents: str = 'No file chosen', button_text: str = 'Load', is_file: bool = True, parent=None):
         super().__init__(parent)
         self.file_path = QtWidgets.QLineEdit('', self)
         self.open_button = QtWidgets.QPushButton(button_text, self)
+        self.is_file = is_file
         self._is_legal = False
 
         self.layout = QtWidgets.QGridLayout(self)
@@ -706,7 +867,10 @@ class PathLineEdit(QtWidgets.QWidget):
         self.layout.addWidget(self.file_path, 1, 1)
 
         self.file_path.textChanged.connect(self._check_legality)
-        self.open_button.clicked.connect(self.choose_file)
+        if self.is_file:
+            self.open_button.clicked.connect(self.choose_file)
+        else:
+            self.open_button.clicked.connect(self.choose_folder)
 
         self.file_path.setText(contents)
 
@@ -719,7 +883,8 @@ class PathLineEdit(QtWidgets.QWidget):
 
     def _check_legality(self):
         current_path = self.file_path.text()
-        if validation.is_legal_file_path(current_path):
+        if (self.is_file and validation.is_legal_file_path(current_path)) or (
+            not self.is_file and validation.is_legal_dir_path(current_path)):
             self._is_legal = True
         else:
             self._is_legal = False
@@ -746,11 +911,15 @@ class PathLineEdit(QtWidgets.QWidget):
         super().setDisabled(to_disable)
 
     def choose_file(self):
-        filename, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Choose a file", str(Path.home()),
-                                                            "All Files (*)")
+        filename, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Choose a file")
         if filename:
             self.file_path.setText(filename)
 
+    def choose_folder(self):
+        dirname = QtWidgets.QFileDialog.getExistingDirectory(self, "Choose a folder")
+        if dirname:
+            self.file_path.setText(dirname)
+
     def text(self):
         return self.file_path.text()
 
@@ -920,6 +1089,92 @@ class OptionalDoubleSpinBox(OptionalSpinBox):
         self.spinbox.setSingleStep(step)
 
 
+class ComparisonPicker(QtWidgets.QWidget):
+    def __init__(self, design_mat: pd.DataFrame, parent=None):
+        super().__init__(parent)
+        self.design_mat = design_mat
+        self.layout = QtWidgets.QHBoxLayout(self)
+        self.factor = QtWidgets.QComboBox(self)
+        self.numerator = QtWidgets.QComboBox(self)
+        self.denominator = QtWidgets.QComboBox(self)
+
+        self.init_ui()
+
+    def init_ui(self):
+        self.layout.addWidget(self.factor)
+        self.layout.addWidget(self.numerator)
+        self.layout.addWidget(self.denominator)
+
+        self.factor.currentTextChanged.connect(self.update_combos)
+        self.factor.addItems([str(item) for item in self.design_mat.columns])
+
+    def update_combos(self):
+        this_factor = self.factor.currentText()
+        if this_factor in self.design_mat.columns:
+            options = sorted({str(item) for item in self.design_mat[this_factor]})
+        else:
+            options = ["Select a factor..."]
+        self.numerator.clear()
+        self.denominator.clear()
+
+        self.numerator.addItems(options)
+        self.denominator.addItems(options)
+
+    def get_value(self) -> typing.Tuple[str, str, str]:
+        return self.factor.currentText(), self.numerator.currentText(), self.denominator.currentText()
+
+
+class ComparisonPickerGroup(QtWidgets.QWidget):
+    def __init__(self, design_mat: pd.DataFrame, parent=None):
+        super().__init__(parent)
+        self.design_mat = design_mat
+        self.widgets = {}
+        self.layout = QtWidgets.QGridLayout(self)
+
+        self.inputs = []
+        self.input_labels = []
+
+        self.init_ui()
+        self.add_comparison_widget()
+
+    def init_ui(self):
+        self.widgets['add_widget'] = QtWidgets.QPushButton('Add comparison')
+        self.widgets['add_widget'].clicked.connect(self.add_comparison_widget)
+        self.layout.addWidget(self.widgets['add_widget'], 0, 0, 1, 4)
+
+        self.widgets['remove_widget'] = QtWidgets.QPushButton('Remove comparison')
+        self.widgets['remove_widget'].clicked.connect(self.remove_comparison_widget)
+        self.layout.addWidget(self.widgets['remove_widget'], 1, 0, 1, 4)
+
+        self.layout.addWidget(QtWidgets.QLabel('<b>Design factor</b>', self), 2, 1)
+        self.layout.addWidget(QtWidgets.QLabel('<b>Numerator</b>', self), 2, 2)
+        self.layout.addWidget(QtWidgets.QLabel('<b>Denominator</b>', self), 2, 3)
+
+    @QtCore.pyqtSlot()
+    def add_comparison_widget(self):
+        self.inputs.append(ComparisonPicker(self.design_mat, self))
+        self.layout.addWidget(self.inputs[-1], len(self.inputs) + 2, 1, 1, 3)
+
+        self.input_labels.append(QtWidgets.QLabel(f'Comparison #{len(self.inputs)}:', self.inputs[-1]))
+        self.layout.addWidget(self.input_labels[-1], len(self.inputs) + 2, 0)
+
+        self.layout.setRowStretch(len(self.inputs) + 2, 1)
+        self.layout.setRowStretch(len(self.inputs) + 3, 2)
+
+    @QtCore.pyqtSlot()
+    def remove_comparison_widget(self):
+        if len(self.inputs) > 0:
+            self.inputs.pop(-1).deleteLater()
+            self.input_labels.pop(-1).deleteLater()
+            self.layout.setRowStretch(len(self.inputs) + 3, 2)
+
+    def get_comparison_values(self):
+        values = []
+        for widget in self.inputs:
+            values.append(widget.get_value())
+        return values
+
+
 class QMultiInput(QtWidgets.QPushButton):
     IS_MULTI_INPUT = True
     CHILD_QWIDGET = None
@@ -935,6 +1190,10 @@ class QMultiInput(QtWidgets.QPushButton):
         self.clicked.connect(self.start_dialog)
         self.init_dialog_ui()
 
+    def clear(self):
+        while len(self.dialog_widgets['inputs']) > 0:
+            self.remove_widget()
+
     @QtCore.pyqtSlot()
     def start_dialog(self):
         self.dialog_widgets['box'].exec()
@@ -974,8 +1233,9 @@ class QMultiInput(QtWidgets.QPushButton):
 
     @QtCore.pyqtSlot()
     def remove_widget(self):
-        self.dialog_widgets['inputs'].pop(-1).deleteLater()
-        self.dialog_widgets['input_labels'].pop(-1).deleteLater()
+        if len(self.dialog_widgets['inputs']) > 0:
+            self.dialog_widgets['inputs'].pop(-1).deleteLater()
+            self.dialog_widgets['input_labels'].pop(-1).deleteLater()
 
     def get_values(self):
         values = []
@@ -1083,6 +1343,53 @@ class QMultiBoolComboBox(QMultiComboBox):
         self.dialog_widgets['inputs'][ind].setCurrentText(str(val))
 
 
+class OptionalMultiInput(QtWidgets.QWidget):
+    MULTI_WIDGET_TYPE = QtWidgets.QWidget
+    IS_MULTI_INPUT = True
+
+    def __init__(self, label: str, text='Set input', parent=None):
+        super().__init__(parent)
+        self.layout = QtWidgets.QHBoxLayout(self)
+        self.multi_widget = self.MULTI_WIDGET_TYPE(label, text, self)
+        self.checkbox = QtWidgets.QCheckBox('Disable input?')
+        self.checkbox.toggled.connect(self.multi_widget.setDisabled)
+        self.checkbox.toggled.connect(self.disable_value_changed)
+        self.valueChanged = self.multi_widget.valueChanged
+
+        self.layout.addWidget(self.checkbox)
+        self.layout.addWidget(self.multi_widget)
+
+    def disable_value_changed(self):
+        self.valueChanged.emit()
+
+    def clear(self):
+        self.checkbox.setChecked(False)
+        self.multi_widget.clear()
+
+    def set_defaults(self, defaults: typing.Union[typing.Iterable, None]):
+        if defaults is None:
+            self.checkbox.setChecked(True)
+        else:
+            self.multi_widget.set_defaults(defaults)
+
+    def get_values(self):
+        if self.checkbox.isChecked():
+            return None
+        return self.multi_widget.get_values()
+
+
+class OptionalMultiLineEdit(OptionalMultiInput):
+    MULTI_WIDGET_TYPE = QMultiLineEdit
+
+
+class OptionalMultiSpinBoxEdit(OptionalMultiInput):
+    MULTI_WIDGET_TYPE = QMultiSpinBox
+
+
+class OptionalMultiDoubleSpinBoxEdit(OptionalMultiInput):
+    MULTI_WIDGET_TYPE = QMultiDoubleSpinBox
+
+
 class ThreadStdOutStreamTextQueueReceiver(QtCore.QObject):
     queue_stdout_element_received_signal = QtCore.pyqtSignal(str)
 
@@ -1180,7 +1487,26 @@ def param_to_widget(param, name: str,
             for action in actions_to_connect:
                 widget.valueChanged.connect(action)
 
-    elif name in {'samples', 'sample_grouping'} and (not pipeline_mode):
+    elif typing_extensions.get_origin(param.annotation) == typing.Union and typing_extensions.Literal in [
+        typing_extensions.get_origin(ann) for ann in typing_extensions.get_args(param.annotation)]:
+        args = typing_extensions.get_args(param.annotation)
+        literal_ind = [typing_extensions.get_origin(ann) for ann in args].index(
+            typing_extensions.Literal)
+        literal = args[literal_ind]
+        without_literal = tuple(args[0:literal_ind] + args[literal_ind + 1:])
+        if param.default in typing_extensions.get_args(literal):
+            this_default = param.default
+            other_default = inspect._empty
+        else:
+            this_default = None
+            other_default = param.default
+        widget = ComboBoxOrOtherWidget(typing_extensions.get_args(literal),
+                                       param_to_widget(NewParam(typing.Union[without_literal], other_default), name,
+                                                       actions_to_connect, pipeline_mode), this_default)
+        for action in actions_to_connect:
+            widget.currentIndexChanged.connect(action)
+
+    elif name in {'samples', 'sample_grouping','replicate_grouping'} and (not pipeline_mode):
         widget = TableColumnGroupPicker()
         for action in actions_to_connect:
             widget.valueChanged.connect(action)
@@ -1223,24 +1549,6 @@ def param_to_widget(param, name: str,
         widget = QtWidgets.QLineEdit(param.default if is_default else '')
         for action in actions_to_connect:
             widget.textChanged.connect(action)
-    elif typing_extensions.get_origin(param.annotation) == typing.Union and typing_extensions.Literal in [
-        typing_extensions.get_origin(ann) for ann in typing_extensions.get_args(param.annotation)]:
-        args = typing_extensions.get_args(param.annotation)
-        literal_ind = [typing_extensions.get_origin(ann) for ann in args].index(
-            typing_extensions.Literal)
-        literal = args[literal_ind]
-        without_literal = tuple(args[0:literal_ind] + args[literal_ind + 1:])
-        if param.default in typing_extensions.get_args(literal):
-            this_default = param.default
-            other_default = inspect._empty
-        else:
-            this_default = None
-            other_default = param.default
-        widget = ComboBoxOrOtherWidget(typing_extensions.get_args(literal),
-                                       param_to_widget(NewParam(typing.Union[without_literal], other_default), name,
-                                                       actions_to_connect, pipeline_mode), this_default)
-        for action in actions_to_connect:
-            widget.currentIndexChanged.connect(action)
     elif typing_extensions.get_origin(param.annotation) == typing_extensions.Literal:
         widget = QtWidgets.QComboBox()
         widget.addItems(typing_extensions.get_args(param.annotation))
@@ -1273,6 +1581,13 @@ def param_to_widget(param, name: str,
             widget.set_defaults(param.default)
         for action in actions_to_connect:
             widget.valueChanged.connect(action)
+    elif param.annotation in (typing.Union[None, str, typing.List[str]], typing.Union[None, typing.List[str]]):
+        widget = OptionalMultiLineEdit(name)
+        if is_default:
+            widget.set_defaults(param.default)
+        for action in actions_to_connect:
+            widget.valueChanged.connect(action)
+
     elif param.annotation in (typing.Union[float, typing.List[float]],
                               typing.Union[float, typing.Iterable[float]]):
         widget = QMultiDoubleSpinBox(name)
@@ -1308,7 +1623,8 @@ def param_to_widget(param, name: str,
         for action in actions_to_connect:
             widget.valueChanged.connect(action)
     elif param.annotation in (Path, typing.Union[str, Path], typing.Union[None, str, Path]):
-        widget = PathLineEdit()
+        is_file = 'folder' not in name
+        widget = PathLineEdit(is_file=is_file)
         for action in actions_to_connect:
             widget.textChanged.connect(action)
         if is_default:
