@@ -642,6 +642,8 @@ class ClusteringRunnerWithNClusters(ClusteringRunner, ABC):
         diff = gap_scores[:-1] - gap_scores[1::] + gap_error[1::]
         # Return all K values that fulfilled the condition as 'potentially good K values'
         k_candidates = n_clusters_range[:-1][diff >= 0]
+        if len(k_candidates) == 0:
+            k_candidates = [1]
         # 1 is not a viable candidate
         if k_candidates[0] == 1:
             k_candidates = k_candidates[1:]
@@ -970,11 +972,11 @@ class CLICOMRunner(ClusteringRunner):
                         'hierarchical': HierarchicalRunner, 'agglomerative': HierarchicalRunner,
                         'hdbscan': HDBSCANRunner}
 
-    def __init__(self, data, power_transform: Union[bool, List[bool]], evidence_threshold: float,
-                 cluster_unclustered_features: bool, min_cluster_size: int, *parameter_dicts: dict,
-                 plot_style: str = 'none',
-                 split_plots: bool = False, ):
+    def __init__(self, data, replicate_grouping: List[List[str]], power_transform: Union[bool, List[bool]],
+                 evidence_threshold: float, cluster_unclustered_features: bool, min_cluster_size: int,
+                 *parameter_dicts: dict, plot_style: str = 'none', split_plots: bool = False, ):
         self.clustering_solutions: list = []
+        self.replicate_grouping = replicate_grouping
         self.evidence_threshold = evidence_threshold
         self.cluster_unclustered_features = cluster_unclustered_features
         self.min_cluster_size = min_cluster_size
@@ -1039,17 +1041,20 @@ class CLICOMRunner(ClusteringRunner):
                                                *[parsing.data_to_list(val) for val in params.values()])
             for setup in setups_to_test:
                 kwargs = {key: val for key, val in zip(itertools.chain(['power_transform'], params.keys()), setup)}
-                try:
-                    runner_class(self.data, **kwargs)
-                    valid_setups.append((runner_class, kwargs))
-                except AssertionError:
-                    continue
+                for replicate in self.replicate_grouping:
+                    args = (self.data[replicate],)
+                    try:
+                        runner_class(*args, **kwargs)
+                        valid_setups.append((runner_class, args, kwargs))
+                    except AssertionError:
+                        continue
 
         return valid_setups
 
-    def run_clustering_setup(self, setup) -> list:
-        runner_class, kwargs = setup
-        runner = runner_class(self.data, **kwargs)
+    @staticmethod
+    def run_clustering_setup(setup) -> list:
+        runner_class, args, kwargs = setup
+        runner = runner_class(*args, **kwargs)
         clusterers = runner.run(plot=False)
         return [clusterer.labels_ for clusterer in clusterers]
 
