@@ -11,6 +11,11 @@ def is_legal_file_path(file_path: str):
     return pth.exists() and pth.is_file()
 
 
+def is_legal_dir_path(dir_path: str):
+    pth = Path(dir_path)
+    return pth.exists() and pth.is_dir()
+
+
 def check_is_df_like(inp):
     """
     checks whether an input file is a pandas DataFrame, a string that represent a path of a .csv file, a Path object \
@@ -55,11 +60,16 @@ def isinstanceinh(obj, parent_class):
 
     :param obj: object to be tested
     :type obj: Any
-    :param parent_class: class to be checked against
-    :type parent_class: type (e.g. list, tuple, int, bool)
+    :param parent_class: class to be checked against, or tuple of classes to be checked against
+    :type parent_class: type (e.g. list, tuple, int, bool), or tuple of types
     :return: True if 'obj' is an instance of 'parent_class' or one of its subclasses, and False otherwise.
     :rtype: bool
     """
+    if isinstance(parent_class, tuple):
+        for cls in parent_class:
+            if issubclass(obj.__class__, cls):
+                return True
+        return False
     return True if issubclass(obj.__class__, parent_class) else False
 
 
