@@ -1,3 +1,5 @@
+import os
+
 import pytest
 
 import rnalysis.utils.io
@@ -550,8 +552,9 @@ def test_save_gene_set(gene_set, expected_split):
             pass
 
 
-def test_kegg_annotation_iterator_api():
-    _ = KEGGAnnotationIterator(6239)
+@pytest.mark.parametrize("args", [(6239,), (6239, 'all'), (6239, ['id1', 'id2'])])
+def test_kegg_annotation_iterator_api(args):
+    _ = KEGGAnnotationIterator(*args)
 
 
 @pytest.mark.parametrize('arguments,url_truth', [('argument', 'https://rest.kegg.jp/operation/argument'),
@@ -617,6 +620,18 @@ def test_kegg_annotation_iterator_get_pathways(monkeypatch):
     assert kegg.get_pathways() == truth
 
 
+def test_kegg_annotation_iterator_get_custom_pathways(monkeypatch):
+    truth = {'path:cel00010': None, 'path:cel00030': None, 'path:cel00051': None}
+
+    def mock_kegg_request(self, operation, arguments):
+        return 'cel', False
+
+    monkeypatch.setattr(KEGGAnnotationIterator, '_kegg_request', mock_kegg_request)
+
+    kegg = KEGGAnnotationIterator(6239, [i for i in truth.keys()])
+    assert kegg.pathway_names == truth
+
+
 def test_kegg_annotation_iterator_get_pathway_annotations(monkeypatch):
     truth = {'path:cel00010': ['Glycolysis / Gluconeogenesis - Caenorhabditis elegans (nematode)',
                                {'cel:CELE_F14B4.2', 'cel:CELE_Y87G2A.8', 'cel:CELE_C50F4.2', 'cel:CELE_Y71H10A.1'}],
@@ -741,3 +756,59 @@ def test_get_gui_videos(monkeypatch):
         assert file_count == len(video_files)
     finally:
         shutil.rmtree(mock_dir, ignore_errors=True)
+
+
+class MockProcess:
+    def __init__(self, returncode: int):
+        self.stdout = [b'things', b'to', b'print']
+        self.returncode = returncode
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return
+
+
+@pytest.mark.parametrize('r_path,expected', [
+    ('auto', ['Rscript', "tests/test_files/test_r_script.R"]),
+    ('D:/Program Files/R', ["D:/Program Files/R/bin/Rscript", "tests/test_files/test_r_script.R"])
+])
+def test_run_r_script(monkeypatch, r_path, expected):
+    script_path = 'tests/test_files/test_r_script.R'
+
+    def mock_popen(process, stdout, stderr, shell=False):
+        assert process == expected
+        return MockProcess(0)
+
+    monkeypatch.setattr(subprocess, 'Popen', mock_popen)
+    monkeypatch.setattr(subprocess, 'run', lambda *args, **kwargs: MockProcess(0))
+
+    run_r_script(script_path, r_path)
+
+
+def test_run_r_script_not_installed(monkeypatch):
+    def mock_run(process, stdout, stderr, shell=False):
+        return MockProcess(1)
+
+    monkeypatch.setattr(subprocess, 'run', mock_run)
+    with pytest.raises(FileNotFoundError):
+        run_r_script('tests/test_files/test_r_script.R')
+
+
+@pytest.mark.parametrize("stdout", [True, False])
+@pytest.mark.parametrize("stderr", [True, False])
+def test_run_subprocess(stdout, stderr):
+    run_subprocess(['python3', '--version'], stdout, stderr)
+
+
+@pytest.mark.parametrize('this_version,response,expected', [
+    ('3.0.0', MockResponse(status_code=503), False),
+    ('3.0.0', MockResponse(text='{"info":{"author":"Guy Teichman","version":"3.1.0"}}'), True),
+    ('3.1.1', MockResponse(text='{"info":{"author":"Guy Teichman","version":"3.1.0"}}'), False),
+    ('3.1.1', MockResponse(text='{"info":{"author":"Guy Teichman","version":"3.1.1"}}'), False),
+])
+def test_is_rnalysis_outdated(monkeypatch, this_version, response, expected):
+    monkeypatch.setattr(requests, 'get', lambda *args, **kwargs: response)
+    monkeypatch.setattr(rnalysis.utils.io, '__version__', this_version)
+    assert is_rnalysis_outdated() == expected
